@@ -731,7 +731,11 @@ __stp_call_mmap_callbacks_with_addr(struct stap_task_finder_target *tgt,
 		length = vma->vm_end - vma->vm_start;
 		offset = (vma->vm_pgoff << PAGE_SHIFT);
 		vm_flags = vma->vm_flags;
+#ifdef STAPCONF_DPATH_PATH
+		dentry = vma->vm_file->f_path.dentry;
+#else
 		dentry = vma->vm_file->f_dentry;
+#endif
 
 		// Allocate space for a path
 		mmpath_buf = _stp_kmalloc(PATH_MAX);
@@ -1245,6 +1249,7 @@ __stp_call_mmap_callbacks_for_task(struct stap_task_finder_target *tgt,
 			    // get deleted from out under us.
 			    vma_cache_p->f_path = &(vma->vm_file->f_path);
 			    path_get(vma_cache_p->f_path);
+			    vma_cache_p->dentry = vma->vm_file->f_path.dentry;
 #else
 			    // Notice we're increasing the reference
 			    // count for 'dentry' and 'f_vfsmnt'.
@@ -1254,8 +1259,8 @@ __stp_call_mmap_callbacks_for_task(struct stap_task_finder_target *tgt,
 			    dget(vma_cache_p->dentry);
 			    vma_cache_p->f_vfsmnt = vma->vm_file->f_vfsmnt;
 			    mntget(vma_cache_p->f_vfsmnt);
-#endif
 			    vma_cache_p->dentry = vma->vm_file->f_dentry;
+#endif
 			    vma_cache_p->addr = vma->vm_start;
 			    vma_cache_p->length = vma->vm_end - vma->vm_start;
 			    vma_cache_p->offset = (vma->vm_pgoff << PAGE_SHIFT);
