@@ -15,50 +15,6 @@
 
 #include "unwind/unwind.h"
 
-static uleb128_t get_uleb128(const u8 **pcur, const u8 *end)
-{
-	const u8 *cur = *pcur;
-	uleb128_t value = 0;
-	unsigned shift;
-
-	for (shift = 0; cur < end; shift += 7) {
-		if (shift + 7 > 8 * sizeof(value)
-		    && (*cur & 0x7fU) >= (1U << (8 * sizeof(value) - shift))) {
-			cur = end + 1;
-			break;
-		}
-		value |= (uleb128_t)(*cur & 0x7f) << shift;
-		if (!(*cur++ & 0x80))
-			break;
-	}
-	*pcur = cur;
-
-	return value;
-}
-
-static sleb128_t get_sleb128(const u8 **pcur, const u8 *end)
-{
-	const u8 *cur = *pcur;
-	sleb128_t value = 0;
-	unsigned shift;
-
-	for (shift = 0; cur < end; shift += 7) {
-		if (shift + 7 > 8 * sizeof(value)
-		    && (*cur & 0x7fU) >= (1U << (8 * sizeof(value) - shift))) {
-			cur = end + 1;
-			break;
-		}
-		value |= (sleb128_t)(*cur & 0x7f) << shift;
-		if (!(*cur & 0x80)) {
-			value |= -(*cur++ & 0x40) << shift;
-			break;
-		}
-	}
-	*pcur = cur;
-
-	return value;
-}
-
 /* Whether this is a real CIE. Assumes CIE (length) sane. */
 static int has_cie_id(const u32 *cie, int is_ehframe)
 {
@@ -151,111 +107,6 @@ static const u32 *cie_for_fde(const u32 *fde, void *unwind_data,
 	return cie;
 }
 
-/* read an encoded pointer and increment *pLoc past the end of the
- * data read. */
-static unsigned long read_ptr_sect(const u8 **pLoc, const void *end,
-				   signed ptrType, unsigned long textAddr,
-				   unsigned long dataAddr, int user, int compat_task, int tableSize)
-{
-	unsigned long value = 0;
-	union {
-		const u8 *p8;
-		const u16 *p16u;
-		const s16 *p16s;
-		const u32 *p32u;
-		const s32 *p32s;
-		const unsigned long *pul;
-		const unsigned int *pui;
-	} ptr;
-
-	if (ptrType < 0 || ptrType == DW_EH_PE_omit)
-		return 0;
-
-	ptr.p8 = *pLoc;
-	switch (ptrType & DW_EH_PE_FORM) {
-	case DW_EH_PE_data2:
-		if (end < (const void *)(ptr.p16u + 1))
-			return 0;
-		if (ptrType & DW_EH_PE_signed)
-			value = _stp_get_unaligned(ptr.p16s++);
-		else
-			value = _stp_get_unaligned(ptr.p16u++);
-		break;
-	case DW_EH_PE_data4:
-#ifdef CONFIG_64BIT
-
-		/* If the tableSize matches the length of data we're trying to return
-		 * or if specifically set to 0 in the call it means we actually want a
-		 * DW_EH_PE_data4 and not a DW_EH_PE_absptr.  If this is not the case
-		 * then we want to fall through to DW_EH_PE_absptr */
-		if (!compat_task || (compat_task && (tableSize == 4 || tableSize == 0)))
-		{
-			if (end < (const void *)(ptr.p32u + 1))
-				return 0;
-
-			if (ptrType & DW_EH_PE_signed)
-				value = _stp_get_unaligned(ptr.p32s++);
-			else
-				value = _stp_get_unaligned(ptr.p32u++);
-			break;
-		}
-	case DW_EH_PE_data8:
-		BUILD_BUG_ON(sizeof(u64) != sizeof(value));
-#else
-		BUILD_BUG_ON(sizeof(u32) != sizeof(value));
-#endif
-	case DW_EH_PE_absptr:
-		if (compat_task)
-		{
-			if (end < (const void *)(ptr.pui + 1))
-				return 0;
-			value = _stp_get_unaligned(ptr.pui++);
-		} else {
-			if (end < (const void *)(ptr.pul + 1))
-				return 0;
-			value = _stp_get_unaligned(ptr.pul++);
-		}
-
-		break;
-	case DW_EH_PE_leb128:
-		BUILD_BUG_ON(sizeof(uleb128_t) > sizeof(value));
-		value = ptrType & DW_EH_PE_signed ? get_sleb128(&ptr.p8, end)
-		    : get_uleb128(&ptr.p8, end);
-		if ((const void *)ptr.p8 > end)
-			return 0;
-		break;
-	default:
-		return 0;
-	}
-	switch (ptrType & DW_EH_PE_ADJUST) {
-	case DW_EH_PE_absptr:
-		break;
-	case DW_EH_PE_pcrel:
-		value += (unsigned long)*pLoc;
-		break;
-	case DW_EH_PE_textrel:
-		value += textAddr;
-		break;
-	case DW_EH_PE_datarel:
-		value += dataAddr;
-		break;
-	default:
-		return 0;
-	}
-	if ((ptrType & DW_EH_PE_indirect)
-	    && _stp_read_address(value, (unsigned long *)value,
-				 (user ? USER_DS : KERNEL_DS)))
-		return 0;
-	*pLoc = ptr.p8;
-
-	return value;
-}
-
-static unsigned long read_pointer(const u8 **pLoc, const void *end, signed ptrType,
-				  int user, int compat_task)
-{
-	return read_ptr_sect(pLoc, end, ptrType, 0, 0, user, compat_task, 0);
-}
 
 /* Parse FDE and CIE content. Basic sanity checks should already have
    been done start/end/version/id (done by is_fde and cie_for_fde).
@@ -1586,7 +1437,8 @@ static int unwind_frame(struct unwind_context *context,
 			break;
 		}
 	}
-	dbug_unwind(1, "returning 0 (%lx)\n", UNW_PC(frame));
+	dbug_unwind(1, "returning 0 (%llx)\n",
+		    (unsigned long long) UNW_PC(frame));
 	return 0;
 
 copy_failed:
@@ -1617,7 +1469,8 @@ static int unwind(struct unwind_context *context, int user)
 	   values on 64-bit registers. */
 	int compat_task = _stp_is_compat_task();
 
-	dbug_unwind(1, "pc=%lx, %lx", pc, UNW_PC(frame));
+	dbug_unwind(1, "pc=%lx, %llx", pc,
+		    (unsigned long long) UNW_PC(frame));
 
 	if (UNW_PC(frame) == 0)
 		return -EINVAL;
