@@ -42,8 +42,8 @@ using namespace std;
 static void __attribute__ ((noreturn))
 usage (int rc)
 {
-  clog << "Usage: " << program_invocation_short_name
-       << " MODULE [-v] [-c CMD | -x PID] [-o FILE] [-C WHEN] [globalname=value ...] [-V]" << endl
+  cout << "Usage: " << program_invocation_short_name
+       << " MODULE [-v] [-c CMD | -x PID] [-o FILE] [-C WHEN] [globalname=value ...] [-V] [-h]" << endl
        << "-v              Increase verbosity." << endl
        << "-c cmd          Command \'cmd\' will be run and " << program_invocation_short_name << " will" << endl
        << "                exit when it does.  The '_stp_target' variable" << endl
@@ -53,7 +53,8 @@ usage (int rc)
        << "                formats for FILE." << endl
        << "-C WHEN         Enable colored errors. WHEN must be either 'auto'," << endl
        << "                'never', or 'always'. Set to 'auto' by default." << endl
-       << "-V              Show version." << endl;
+       << "-V              Show version." << endl
+       << "-h              Show this help text." << endl;
 
   exit (rc);
 }
@@ -73,7 +74,7 @@ main(int argc, char * const argv[])
 
   // First, option parsing.
   int opt;
-  while ((opt = getopt (argc, argv, "c:x:vwo:VC:")) != -1)
+  while ((opt = getopt (argc, argv, "c:x:vwo:VhC:")) != -1)
     {
       switch (opt)
         {
@@ -98,12 +99,14 @@ main(int argc, char * const argv[])
 	  break;
 
         case 'V':
-          fprintf(stderr, "Systemtap Dyninst loader/runner (version %s/%s, %s)\n"
-                          "Copyright (C) 2012-2014 Red Hat, Inc. and others\n"
-                          "This is free software; see the source for copying conditions.\n",
-                  VERSION, DYNINST_FULL_VERSION, STAP_EXTENDED_VERSION);
+          printf("Systemtap Dyninst loader/runner (version %s/%s, %s)\n"
+                 "Copyright (C) 2012-2014 Red Hat, Inc. and others\n"
+                 "This is free software; see the source for copying conditions.\n",
+                 VERSION, DYNINST_FULL_VERSION, STAP_EXTENDED_VERSION);
           return 0;
-
+        case 'h':
+          usage(0);
+          break;
         case 'C':
           if (!strcmp(optarg, "never"))
             color_errors = false;
