@@ -1,5 +1,5 @@
 // tapset for netfilter hooks
-// Copyright (C) 2012 Red Hat Inc.
+// Copyright (C) 2012-2014 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -187,20 +187,20 @@ netfilter_derived_probe::netfilter_derived_probe (systemtap_session &s, probe* p
         }
       catch (const runtime_error&) 
         {
-          throw semantic_error
+          throw SEMANTIC_ERROR
               (_F("unsupported netfilter priority \"%s\" for protocol family \"%s\"; need stap -g",
               priority.c_str(), pf.c_str()));
         }
 
       // Complain and abort if there were any hook name errors
       if (hook_error)
-            throw semantic_error
+            throw SEMANTIC_ERROR
                 (_F("unsupported netfilter hook \"%s\" for protocol family \"%s\"; need stap -g",
                 hook.c_str(), pf.c_str()));
 
       // Complain and abort if there were any pf errors
       if (pf_error)
-        throw semantic_error
+        throw SEMANTIC_ERROR
             (_F("unsupported netfilter protocol family \"%s\"; need stap -g", pf.c_str()));
     }
 
@@ -232,6 +232,7 @@ netfilter_derived_probe::join_group (systemtap_session& s)
   if (! s.netfilter_derived_probes)
     s.netfilter_derived_probes = new netfilter_derived_probe_group ();
   s.netfilter_derived_probes->enroll (this);
+  this->group = s.netfilter_derived_probes;
 }
 
 
@@ -267,7 +268,13 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       // Previous to kernel 2.6.22, the hookfunction definition takes a struct sk_buff **skb,
       // whereas currently it uses a *skb. We need emit the right version so this will
       // compile on RHEL5, for example.
-      s.op->newline() << "#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)";
+      s.op->newline() << "#ifdef STAPCONF_NETFILTER_V313";
+
+      s.op->newline() << "(const struct nf_hook_ops *nf_ops, struct sk_buff *nf_skb, const struct net_device *nf_in, const struct net_device *nf_out, int (*nf_okfn)(struct sk_buff *))";
+      s.op->newline() << "{";
+
+      s.op->newline() << "#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)";
+
       s.op->newline() << "(unsigned int nf_hooknum, struct sk_buff *nf_skb, const struct net_device *nf_in, const struct net_device *nf_out, int (*nf_okfn)(struct sk_buff *))";
       s.op->newline() << "{";
 
@@ -280,6 +287,9 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       s.op->newline(-1) << "#endif";
       s.op->newline(1) << "const struct stap_probe * const stp = & stap_probes[" << np->session_index << "];";
       s.op->newline() << "int nf_verdict = NF_ACCEPT;"; // default NF_ACCEPT, to be used by $verdict context var
+      s.op->newline() << "#ifdef STAPCONF_NETFILTER_V313";
+      s.op->newline() << "unsigned int nf_hooknum = nf_ops->hooknum;";
+      s.op->newline() << "#endif";
       common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp",
                                      "stp_probe_type_netfilter",
                                      false);
@@ -303,7 +313,7 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       else
         s.op->newline() << "(void) nf_in;";
       if (np->context_vars.find("__nf_out") != np->context_vars.end())
-        s.op->newline() << c_p + "." + s.up->c_localname("__nf_in") + " = (int64_t)(uintptr_t) nf_out;";
+        s.op->newline() << c_p + "." + s.up->c_localname("__nf_out") + " = (int64_t)(uintptr_t) nf_out;";
       else
         s.op->newline() << "(void) nf_out;";
       if (np->context_vars.find("__nf_verdict") != np->context_vars.end())
@@ -314,10 +324,10 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       // Invoke the probe handler
       s.op->newline() << "(*stp->ph) (c);";
 
-      common_probe_entryfn_epilogue (s, false);
+      common_probe_entryfn_epilogue (s, false, otf_safe_context(s));
 
       if (np->context_vars.find("__nf_verdict") != np->context_vars.end())
-        s.op->newline() << "nf_verdict = (int) "+c_p+"." + s.up->c_localname("__nf_verdict") + ";";
+        s.op->newline() << "if (c != NULL) nf_verdict = (int) "+c_p+"." + s.up->c_localname("__nf_verdict") + ";";
 
       s.op->newline() << "return nf_verdict;";
       s.op->newline(-1) << "}";
@@ -397,7 +407,7 @@ netfilter_var_expanding_visitor::visit_target_symbol (target_symbol* e)
       assert(e->name.size() > 0 && e->name[0] == '$');
 
       if (e->addressof)
-        throw semantic_error(_("cannot take address of netfilter hook context variable"), e->tok);
+        throw SEMANTIC_ERROR(_("cannot take address of netfilter hook context variable"), e->tok);
 
       // We map all $context variables to similarly named probe locals.
       // See emit_module_decls for how the parameters & result are handled.
@@ -412,14 +422,14 @@ netfilter_var_expanding_visitor::visit_target_symbol (target_symbol* e)
       else if (e->name == "$verdict") { c_var = "__nf_verdict"; lvalue_ok = true; need_guru = true; }
       // XXX: also support $$vars / $$parms
       else
-        throw semantic_error(_("unsupported context variable"), e->tok);
+        throw SEMANTIC_ERROR(_("unsupported context variable"), e->tok);
 
       if (! lvalue_ok && is_active_lvalue (e))
-        throw semantic_error(_("write to netfilter parameter not permitted"), e->tok);
+        throw SEMANTIC_ERROR(_("write to netfilter parameter not permitted"), e->tok);
 
       // Writing to variables like $verdict requires guru mode, for obvious reasons
       if(need_guru && !sess.guru_mode)
-        throw semantic_error(_("write to netfilter verdict requires guru mode; need stap -g"), e->tok);
+        throw SEMANTIC_ERROR(_("write to netfilter verdict requires guru mode; need stap -g"), e->tok);
 
       context_vars.insert (c_var);
 
@@ -464,10 +474,10 @@ netfilter_builder::build(systemtap_session & sess,
   string priority = "0";      // Default: somewhere in the middle
 
   if(!get_param(parameters, TOK_HOOK, hook))
-    throw semantic_error (_("missing hooknum"));
+    throw SEMANTIC_ERROR (_("missing hooknum"));
 
   if(!get_param(parameters, TOK_PF, pf))
-    throw semantic_error (_("missing protocol family"));
+    throw SEMANTIC_ERROR (_("missing protocol family"));
 
   get_param(parameters, TOK_PRIORITY, priority);
 
