@@ -647,7 +647,7 @@ stap_waitpid(int verbose, pid_t pid)
       spawned_pids.erase(pid);
       ret = WIFEXITED(status) ? WEXITSTATUS(status) : 128 + WTERMSIG(status);
       if (verbose > 1)
-        clog << _F("Spawn waitpid result (0x%x): %d", status, ret) << endl;
+        clog << _F("Spawn waitpid result (0x%x): %d", (unsigned)status, ret) << endl;
     }
   else
     {
@@ -1112,17 +1112,14 @@ bool
 is_valid_pid (pid_t pid, string& err_msg)
 {
   err_msg = "";
-  if (pid <= 0 || kill(pid, 0) == -1)
+  if (pid <= 0)
     {
-      switch (errno) // ignore EINVAL: invalid signal
-      {
-        case ESRCH:
-          err_msg = "pid given does not correspond to a running process";
-        case EPERM:
-          err_msg = "invalid permissions for signalling given pid";
-        default:
-          err_msg = "invalid pid";
-      }
+      err_msg = _F("cannot probe pid %d: Invalid pid", pid);
+      return false;
+    }
+  else if (kill(pid, 0) == -1)
+    {
+      err_msg = _F("cannot probe pid %d: %s", pid, strerror(errno));
       return false;
     }
   return true;
